//////////////////////////////////////////////////////////////////////
// Simple ARM7 stub (sends RTC, TSC, and X/Y data to the ARM 9)
// -- joat
//////////////////////////////////////////////////////////////////////

#include <NDS/NDS.h>

#include <NDS/ARM7/BIOS.h>
#include <NDS/ARM7/touch.h>
#include <NDS/ARM7/clock.h>

#include "shine.h"
#include "do_not.h"
#include "choice_yours.h"
#include "desert_rose.h"
//#include "were_band.h"

//////////////////////////////////////////////////////////////////////

void startSound(int sr, const uint8* data, uint32 bytes)
{
  SOUND_CR &= ~SCHANNEL_ENABLE;
  SCHANNEL_CR(0) &= ~SOUND_ENABLE;
  SCHANNEL_CR(1) &= ~SOUND_ENABLE;
  SOUND_CR = SCHANNEL_ENABLE | SOUND_VOL(0x7F);
  SCHANNEL_TIMER(0) = SOUND_FREQ(sr);
  SCHANNEL_SOURCE(0) = (uint32)data;
  SCHANNEL_LENGTH(0) = bytes;
  SCHANNEL_CR(0) = SOUND_ENABLE | SOUND_VOL(0x7F);
  SCHANNEL_TIMER(1) = SOUND_FREQ(sr);
  SCHANNEL_SOURCE(1) = (uint32)data;
  SCHANNEL_LENGTH(1) = bytes;
  SCHANNEL_CR(1) = SOUND_ENABLE | SOUND_VOL(0x7F);
}

void InterruptHandler(void) {
  int t1, t2;
  static uint32 heartbeat = 0;
  static int last_id = -1;

  if (IF & IRQ_VBLANK) {
    if (heartbeat > 300) {
    if (last_id != IPC->mailRead) {
      switch(IPC->mailRead) {
      case 1:
	startSound(22050, GETRAW(shine), GETRAWSIZE(shine));
	break;
      case 2:
	startSound(11025, GETRAW(do_not), GETRAWSIZE(do_not));
	break;
      case 3:
	startSound(8000, GETRAW(choice_yours), GETRAWSIZE(choice_yours));
	break;
      case 4:
	startSound(6000, GETRAW(desert_rose), GETRAWSIZE(desert_rose));
	break;
      case 5:
// 	startSound(4000, GETRAW(were_band), GETRAWSIZE(were_band));
// 	break;
      case 0:
      default:
	SOUND_CR = 0;
	SCHANNEL_CR(0) = 0;
	SCHANNEL_CR(1) = 0;
      }
      last_id = IPC->mailRead;
    }
    }
    heartbeat++;
    IPC->heartbeat = heartbeat;

    // Read the X/Y buttons and the /PENIRQ line
    IPC->buttons = XKEYS;

    // Read the touch screen
    IPC->touchX = touchRead(TSC_MEASURE_X);
    IPC->touchY = touchRead(TSC_MEASURE_Y);
    IPC->touchZ1 = touchRead(TSC_MEASURE_Z1);
    IPC->touchZ2 = touchRead(TSC_MEASURE_Z2);

    // Read the time
    rtcGetTime((uint8 *)IPC->curtime);
    BCDToInteger((uint8 *)&(IPC->curtime[1]), 7);
 
    // Read the temperature
    IPC->temperature = touchReadTemperature(&t1, &t2);
    IPC->tdiode1 = t1;
    IPC->tdiode2 = t2;
  }

  // Acknowledge interrupts
  IF = IF;
}

//////////////////////////////////////////////////////////////////////

int main(int argc, char ** argv) {
  IPC->mailRead = 0;
  // Reset the clock if needed
  rtcReset();

  // Turn on sound
  powerON(POWER_SOUND);

  // Set up the interrupt handler
  IME = 0;
  IRQ_HANDLER = &InterruptHandler;
  IE = IRQ_VBLANK;
  IF = ~0;
  DISP_SR = DISP_VBLANK_IRQ;
  IME = 1;

  // Keep the ARM7 out of main RAM
  while (1) swiWaitForVBlank();
  return 0;
}

//////////////////////////////////////////////////////////////////////
