
#include "IntroScreen.hpp"
#include <NDS/ipc.h>
#include <NDS/memory.h>
#include <NDS/system.h>
#include <NDS/ARM9/video.h>

#include "PRandL.h"

const int IntroScreen::_tile_base = 0;
const int IntroScreen::_map_base = 31;
const unsigned int IntroScreen::_display_time = 300;

IntroScreen::IntroScreen(const uint16* appData, uint32 sizeof_appData,
			 const uint16* appPalette, uint32 sizeof_appPalette,
			 const uint16* appMap)
  : _main(BOTTOM_SCREEN, _tile_base, _map_base, COLORS_256),
    _sub(TOP_SCREEN, _tile_base, _map_base, COLORS_256),
    update_counter(0), _appMap(appMap)
{

  _main.prepare(appData, sizeof_appData, appPalette, sizeof_appPalette);
  _sub.prepare(PRandLData, sizeof(PRandLData),
	       PRandLPalette, sizeof(PRandLPalette));
}

void IntroScreen::prepare(void)
{
  POWER_CR = POWER_ALL_2D;

  vramSetBankA(VRAM_A_MAIN_BG);
  vramSetBankB(VRAM_B_MAIN_SPRITE);
  vramSetBankC(VRAM_C_SUB_BG);

  _main.load(_appMap);
  _sub.load(PRandLMap);

  DISPLAY_CR = MODE_0_2D | _main.displayCr();
  SUB_DISPLAY_CR = MODE_0_2D | _sub.displayCr();

  update_counter = 0;
}

bool IntroScreen::updateDisplay(void)
{
  bool keep_running = true;
  bool pen_down = (~(IPC->buttons) & (1 << 6)) ? true : false;
  bool start_button = (READ_KEYS & KEY_START);

  ++update_counter;

  if ((update_counter > _display_time) || pen_down || start_button) {
    keep_running = false;
    DISPLAY_CR |= FORCED_BLANK;
    SUB_DISPLAY_CR |= FORCED_BLANK;
  }

  return keep_running;
}
