
#include "KeyPad.hpp"

#include "TitleScreen.hpp"
#include "PRandL.h"
#include "title.h"

#include "instructions.h"
//#include "audio_adrenaline_band.h"
#include "kj52_choice_yours.h"
#include "newsboys_shine.h"
#include "sting_desert_rose.h"
#include "smashups_do_not.h"

#include <NDS/ipc.h>
#include <NDS/memory.h>
#include <NDS/system.h>
#include <NDS/ARM9/video.h>

#define SONGS 4

const int TitleScreen::_tile_base = 0;
const int TitleScreen::_map_base = 31;
    /*                                                                  */
    /* xDotSize = ((raw_x1 - raw_x2) << SCALE_SHIFT) / (dx1 - dx2)      */
    /* x0 = ((raw_x1 + raw_x2) - (dx1 + dx2) * xDotSize) / 2            */
    /* yDotSize = ((raw_y1 - raw_y2) << SCALE_SHIFT) / (dy1 - dy2)      */
    /* y0 = ((raw_y1 + raw_y2) - (dy1 + dy2) * yDotSize) / 2            */
    /*                                                                  */
// (01:00:03) Costis: #define CALIB_DOT_SCALE_SHIFT        8   // X dot size precision
// (01:00:03) Costis: #define CALIB_ORIGIN_SCALE_SHIFT     2   // Origin coordinate precision
// (01:00:15) Costis: for your shift defines, Darkain


TitleScreen::TitleScreen()
  : _main(BOTTOM_SCREEN, _tile_base, _map_base, COLORS_256),
    _sub(TOP_SCREEN, _tile_base, _map_base, COLORS_256),
    _song_data(), _update_counter(0)
{
  _main.prepare(titleData, sizeof(titleData), titlePalette, sizeof(titlePalette));
  _sub.prepare(PRandLData, sizeof(PRandLData), PRandLPalette, sizeof(PRandLPalette));

  _song_data[0]._songData = instructionsData;
  _song_data[1]._songData = newsboys_shineData;
  _song_data[2]._songData = smashups_do_notData;
  _song_data[3]._songData = kj52_choice_yoursData;
  _song_data[4]._songData = sting_desert_roseData;
  //  _song_data[5]._songData = audio_adrenaline_bandData;
}

/* VRAM Bank Control */
#define VRAM_ALL_ENABLE    (1 << 7)
 
//do not combine these defines (other than offset for main display)
#define VRAM_ALL_LCD       0
#define VRAM_ABCD_BG       1
#define VRAM_ABCD_OBJ      2
#define VRAM_ABCD_TEXTURE  3

void TitleScreen::prepare(void)
{
  uint32 i = 0, j = 0;

  POWER_CR = POWER_ALL_2D;

  vramSetMainBanks(VRAM_A_MAIN_BG, VRAM_B_MAIN_SPRITE,
		   VRAM_C_SUB_BG, VRAM_D_SUB_SPRITE);

  reset();

  _sub.load(PRandLMap);
  _main.load(titleMap);

  initOAM();

  for(i = 0; i < SONGS + 1; ++i) {
    for(j = 0; j < 8; ++j) {
      int offset = 16 * 16 * j;
      LoadSpriteData1D(1, &_song_data[i]._songData[offset], 16 * 16,
		       &_song_data[i]._spr[j].gfxID);
      _song_data[i]._spr[j].oam = &OAMCopy[j + i * 8];
      _song_data[i]._spr[j].configureOAM(_song_data[i]._spr[j].gfxID,
					 256, 32, 16);
      _song_data[i]._spr[j].y = SCREEN_HEIGHT;
      _song_data[i]._spr[j].x = j * 32;
      MoveSprite(&_song_data[i]._spr[j]);
    }
  }

  for(j = 0; j < 8; j++) {
    _song_data[0]._spr[j].y = SCREEN_HEIGHT - 24;
    MoveSprite(&_song_data[0]._spr[j]);
  }

  for(i = 0; i < sizeof(newsboys_shinePalette) / sizeof(uint16); ++i) {
    SPRITE_PALETTE[i] = newsboys_shinePalette[i];
  }

  DISPLAY_CR = MODE_0_2D | _main.displayCr() | displayCr();
  SUB_DISPLAY_CR = MODE_0_2D | _sub.displayCr();

  updateOAM();

  IPC->mailRead = 0;

  _update_counter = 0;
}

bool TitleScreen::updateDisplay(void)
{
  static KeyPad kp(20, 5);
  uint16 keypress = kp.Update();
  bool result = true;
  uint16 old_song = IPC->mailRead;
  uint16 new_song(old_song);

  if (keypress & KEY_RIGHT) {
    ++new_song;
    if (new_song > SONGS) {
      new_song = 0;
    }
  } else if (keypress & KEY_LEFT) {
    --new_song;
    // Wrap around of an unsigned number.
    if (new_song > 10) {
      new_song = SONGS;
    }
  }
  if (new_song != old_song) {
    // Move the old sprite off the screen and put the new one on.
    for(int j = 0; j < 8; ++j) {
      _song_data[old_song]._spr[j].y = SCREEN_HEIGHT;
      MoveSprite(&_song_data[old_song]._spr[j]);    

      _song_data[new_song]._spr[j].y = SCREEN_HEIGHT - 24;
      MoveSprite(&_song_data[new_song]._spr[j]);    
      updateOAM();
    }

    IPC->mailRead = new_song;
  }

  return result;
}
