//////////////////////////////////////////////////////////////////////
// Simple ARM9 demo (touch screen controls the colors)
// -- joat
//////////////////////////////////////////////////////////////////////

typedef unsigned short int u16;

#include "TitleScreen.hpp"

#include <NDS/NDS.h>
#include <NDS/ARM9/BIOS.h>
#include <NDS/ARM9/video.h>

#define BLACK  RGB15( 0,  0,  0)
#define RED    RGB15(31,  0,  0)
#define GREEN  RGB15( 0, 31,  0)
#define BLUE   RGB15( 0,  0, 31)
#define PINK   RGB15(31,  0, 31)
#define PURPLE RGB15(15,  0, 15)
#define TURQ   RGB15( 0, 31, 31)
#define YELLOW RGB15(31, 31,  0)
#define ORANGE RGB15(31, 15,  0)
#define GREY   RGB15(15, 15, 15)
#define WHITE  RGB15(31, 31, 31)

class AudioSamplerMain
{
public:
  static AudioSamplerMain& theApp(void);
  void go(void);
  void updateDisplay(void);

private:
  AudioSamplerMain();
  AudioSamplerMain(const AudioSamplerMain& s);
  AudioSamplerMain& operator=(const AudioSamplerMain& s);

  void runTitle(void);

  virtual ~AudioSamplerMain();

  TitleScreen title;
};

void InterruptHandler(void) {
  if (IF & IRQ_VBLANK) {
    AudioSamplerMain& app(AudioSamplerMain::theApp());

    app.updateDisplay();

    IF = IRQ_VBLANK;
  }
}

AudioSamplerMain::AudioSamplerMain()
  : title()
{}

AudioSamplerMain::AudioSamplerMain(const AudioSamplerMain& s)
  : title()
{}

AudioSamplerMain& AudioSamplerMain::operator=(const AudioSamplerMain& s)
{
  return *this;
}

AudioSamplerMain::~AudioSamplerMain()
{}

void AudioSamplerMain::runTitle(void)
{
  title.updateDisplay();
}

void AudioSamplerMain::updateDisplay(void) {
  runTitle();
}

/* VRAM Bank Control */
#define VRAM_ALL_ENABLE    (1 << 7)
 
//do not combine these defines (other than offset for main display)
#define VRAM_ALL_LCD       0
#define VRAM_ABCD_BG       1
#define VRAM_ABCD_OBJ      2
#define VRAM_ABCD_TEXTURE  3

AudioSamplerMain& AudioSamplerMain::theApp(void)
{
  static AudioSamplerMain app;
  return app;
}

void AudioSamplerMain::go(void) {
  title.prepare();

  // Enable the V-blank interrupt
  IME = 0;
  IRQ_HANDLER = &InterruptHandler;
  IE = IRQ_VBLANK;
  IF = ~0;
  DISP_SR = DISP_VBLANK_IRQ;
  IME = 1;

  while (1) {
    swiWaitForVBlank();
  }
}

int main(int argc, char** argv) {
  AudioSamplerMain& app(AudioSamplerMain::theApp());

  app.go();

  return 0;
}


//////////////////////////////////////////////////////////////////////
